@@ -9,10 +9,11 @@ from metaindex.humanizer import humanize
 import metaindex.cache
 import metaindex.indexer
 
-from cursedspace import Key, InputLine, ShellContext
+from cursedspace import Key, InputLine, ShellContext, Completion, colors
 
 from metaindexmanager import clipboard
 from metaindexmanager import command
+from metaindexmanager.shared import DICTIONARY_SCOPE
 from metaindexmanager.utils import logger, first_line
 from metaindexmanager.panel import ListPanel
 from metaindexmanager.docpanel import DocPanel
@@ -54,14 +55,57 @@ class Line(LineBase):
         return super().comparator() + [self.tag.lower(), str(self.value).lower()]
 
 
+class EditorLineCompletion(Completion):
+    COLOR = colors.BLUE
+
+    def __init__(self, editorline, words):
+        super().__init__(editorline)
+        self.words = words
+
+    def update(self, y, x):
+        text = self.inputline.text[:self.inputline.cursor]
+        if text.endswith(' '):
+            text = ''
+        else:
+            text = text.split(' ')[-1]
+
+        options = [word
+                   for word in sorted(self.words, key=lambda a: str(a))
+                   if str(word).startswith(text)]
+        if len(options) == 1 and str(options[0]) == text:
+            # only available option typed out
+            self.close()
+            self.app.paint()
+        elif len(options) > 0:
+            # there are options? show them
+            self.set_alternatives(options, (y, x))
+            self.app.paint()
+        elif self.is_visible:
+            # no options, but the box is still visible? close it
+            self.close()
+            self.app.paint()
+
+    def handle_key(self, key):
+        result = super().handle_key(key)
+        if result and self.suggestions is None:
+            self.inputline.paint(True)
+        return result
+
+
 class EditorLine(InputLine):
-    def __init__(self, panel, text=None):
+    def __init__(self, panel, text=None, suggestions=None):
         y, x, _, _ = panel.content_area()
         y += panel.pos[0] + panel.cursor - panel.offset
         x += panel.pos[1] + panel.columns[0]
         if text is None:
             text = panel.app.as_printable(panel.selected_line.value, None)
-        super().__init__(panel.app, panel.columns[1], (y, x), text=text, background='░')
+        super().__init__(panel.app,
+                         panel.columns[1],
+                         (y, x),
+                         text=text,
+                         background='░')
+        if len(suggestions) > 0:
+            self.completion = EditorLineCompletion(self, suggestions)
 
         self.item = panel.selected_line
         self.parent = panel
@@ -72,7 +116,16 @@ class EditorLine(InputLine):
         logger.debug(f"Enter tag edit mode for {self.item} (text: '{text}')")
 
     def handle_key(self, key):
-        if key in [Key.ESCAPE, "^C"]:
+        if key in [Key.TAB, "^N"] and \
+           (self.completion is None or not self.completion.is_visible):
+            self.update_completion()
+
+        elif self.completion is not None and \
+             self.completion.is_visible and \
+             self.completion.handle_key(key):
+            pass
+
+        elif key in [Key.ESCAPE, "^C"]:
             self.destroy()
 
         elif key in [Key.RETURN, "^S"]:
@@ -175,6 +228,11 @@ class EditorPanel(ListPanel):
             return
         self.app.open_file(self.item)
 
+    def open_selected_with(self, cmd):
+        if self.item is None:
+            return
+        self.app.open_with(self.item, cmd)
+
     def changed(self, change):
         logger.debug(f"Added change to stack: {change}")
         self.changes = self.changes[:self.change_ptr]
@@ -252,7 +310,22 @@ class EditorPanel(ListPanel):
            (text is None and '\n' in str(self.selected_line.value.raw_value)):
             self.multiline_edit()
         else:
-            self.editor = EditorLine(self, text=text)
+            tag = self.selected_line.prefix + "." + self.selected_line.tag
+            words = set(self.app.configuration.list(DICTIONARY_SCOPE,
+                                                    self.selected_line.tag,
+                                                    ''))
+            use_all_values = len(words) == 0
+            if len(words) > 0:
+                if '*' in words:
+                    use_all_values = True
+                    words.remove('*')
+            if use_all_values:
+                words |= set(sum([[str(v.raw_value) for v in result[tag]]
+                                  for result in self.app.cache.find(tag + "?")],
+                                 start=[]))
+            self.editor = EditorLine(self,
+                                     text=text,
+                                     suggestions=words)
             self.app.paint(True)
 
     def cancel_edit(self):
@@ -459,6 +532,7 @@ class EditorPanel(ListPanel):
 
     def do_paint_item(self, y, x, maxwidth, is_selected, item):
         if isinstance(item, Header):
+            self.win.addstr(y, x, " "*maxwidth)
             self.win.addstr(y, x, item.title[:self.dim[1]-x-2], curses.A_BOLD)
         else:
             for colidx, text in enumerate([item.tag, item.value]):
