@@ -4,12 +4,23 @@ import os
 import shutil
 import threading
 import subprocess
+import stat
+import datetime
+try:
+    import pwd
+except ImportError:
+    pwd = None
+try:
+    import grp
+except ImportError:
+    grp = None
 
 from multidict import MultiDict
 
 from cursedspace import Panel, colors, ShellContext, Completion
 
 from metaindex import stores
+from metaindex.humanizer import format_filesize, format_datetime
 from metaindex.configuration import SECTION_GENERAL, CONFIG_COLLECTION_METADATA
 
 from metaindexmanager import command
@@ -72,10 +83,17 @@ class FolderObserver(threading.Thread):
         self.active = True
         text = None
         while self.active:
-            items = list(self.path.iterdir())
-            newtext = "\n".join([str(i) for i in sorted(items)])
-            if text is None or text != newtext and self.active:
-                text = newtext
+            try:
+                items = list(self.path.iterdir())
+            except OSError as exc:
+                items = exc
+
+            if isinstance(items, list):
+                newtext = "\n".join([str(i) for i in sorted(items)])
+                if text is None or text != newtext and self.active:
+                    text = newtext
+                    self.panel.app.callbacks.put((self.panel, lambda: self.on_change(items)))
+            else:
                 self.panel.app.callbacks.put((self.panel, lambda: self.on_change(items)))
             self.do_interrupt.wait(2)  # TODO: should be configurable
             self.do_interrupt.clear()
@@ -87,7 +105,9 @@ class FolderObserver(threading.Thread):
     def on_change(self, items):
         """Called when the content of the currently watched folder changes"""
         if self.active:
-            return self.panel.finish_change_path([Line(self.panel, i) for i in items])
+            if isinstance(items, list):
+                return self.panel.finish_change_path([Line(self.panel, i) for i in items])
+            self.panel.change_path_failed(items)
         return False
 
     def close(self):
@@ -102,6 +122,13 @@ class FilePanel(ListPanel):
     SCOPE = 'files'
 
     CONFIG_ICONS = 'use-icons'
+    CONFIG_SELECT = 'selection-icon'
+    CONFIG_INFO = 'info'
+
+    INFO_COLUMNS = {'size', 'bytes', 'owner', 'group',
+                    'rights', 'mode',
+                    'num_rights', 'octmode',
+                    'lm_abs', 'lm_duration'}
 
     def __init__(self, application, path='.'):
         super().__init__(application)
@@ -113,6 +140,10 @@ class FilePanel(ListPanel):
         self.fancy_icons = False
         self.observer = None
         self.cursor_cache = {}
+        self.select_icon = ' '
+        self.info_columns = []
+        self.info_columns_width = []
+        self.info_columns_content = {}
         self.configuration_changed()
         self.change_path(path)
 
@@ -132,6 +163,24 @@ class FilePanel(ListPanel):
             changed = self.fancy_icons != value
             self.fancy_icons = value
 
+        if name is None or name == self.CONFIG_SELECT:
+            value = self.app.configuration.get(self.SCOPE, self.CONFIG_SELECT, ' ')
+            if len(value) < 1:
+                value = ' '
+            else:
+                value = value[0]
+            changed = value != self.select_icon
+            self.select_icon = value
+
+        if name is None or name == self.CONFIG_INFO:
+            values = [c
+                      for c in self.app.configuration.list(self.SCOPE, self.CONFIG_INFO, '')
+                      if c in self.INFO_COLUMNS]
+            changed = values != self.info_columns
+            self.info_columns = values
+            if changed:
+                self.calculate_info_columns()
+
         if changed:
             if self.win is not None:
                 self.scroll()
@@ -169,6 +218,7 @@ class FilePanel(ListPanel):
                     self.cursor = nr
                     break
 
+        self.calculate_info_columns()
         self.scroll()
         # drop elements from the multi selection that are no longer there
         self.multi_selection = [item for item in self.multi_selection if item in self.items]
@@ -181,11 +231,19 @@ class FilePanel(ListPanel):
 
         return True
 
+    def change_path_failed(self, exc):
+        self.app.error(str(exc))
+        if self.observer is not None:
+            self.observer.close()
+        self.observer = None
+
     def title(self):
         return str(self.path)
 
     @property
     def selected_path(self):
+        if self.selected_item is None:
+            return None
         return self.selected_item.fullpath
 
     @property
@@ -205,6 +263,14 @@ class FilePanel(ListPanel):
         else:
             self.app.open_file(item.fullpath)
 
+    def open_selected_with(self, cmd):
+        item = self.selected_item
+
+        if item is None:
+            self.app.error("Nothing selected")
+            return
+        self.app.open_with(item.fullpath, cmd)
+
     def on_clipboard_item_use(self, item):
         """Called when a clipboard item copied/cut from here is being pasted"""
         if item.extra != 'CUT':
@@ -510,10 +576,11 @@ class FilePanel(ListPanel):
 
         icon = '' if not self.fancy_icons else item.icon + " "
 
-        text = f"{icon}{item.name}"
+        text = f" {icon}{item.name}"
         if item in self.multi_selection:
-            text = "✔ " + text
+            text = self.select_icon + " " + text
             attr |= curses.A_BOLD
+
         try:
             self.win.addstr(y, x, " "*maxwidth, attr)
         except curses.error:
@@ -523,12 +590,36 @@ class FilePanel(ListPanel):
         except curses.error:
             pass
 
+        if len(self.info_columns) == 0:
+            return
+        cut_off = x + 2*maxwidth//3
+        for column in range(len(self.info_columns)):
+            column = len(self.info_columns) - column - 1
+            content = self.info_columns_content[item.path][column]
+
+            ix = x + maxwidth - sum(self.info_columns_width[column:])
+            ix += self.info_columns_width[column] - len(content)
+            if ix < cut_off:
+                break
+            try:
+                self.win.addstr(y, ix, content, attr)
+            except curses.error:
+                pass
+
     def handle_scrolling_key(self, key):
         handled, must_repaint, must_clear = super().handle_scrolling_key(key)
         if handled:
             self.cursor_cache[self.path] = self.cursor
         return handled, must_repaint, must_clear
 
+    def line_matches_find(self, cursor):
+        if self.items is None or self.find_text is None:
+            return False
+        item = self.items[cursor]
+        if self.app.configuration.find_is_case_sensitive:
+            return self.find_text in item.name
+        return self.find_text.lower() in item.name.lower()
+
     def jump_to(self, item, path=None):
         logger.debug(f"Jump to {path}/{item}")
         if path is None:
@@ -571,6 +662,71 @@ class FilePanel(ListPanel):
         if self.observer is not None:
             self.observer.close()
 
+    def calculate_info_columns(self):
+        self.info_columns_width = [0]*len(self.info_columns)
+        self.info_columns_content = {}
+
+        if self.items is None or len(self.items) == 0:
+            return
+
+        for item in self.items:
+            self.info_columns_content[item.path] = ["?"]*len(self.info_columns)
+            for column, content in enumerate(self.info_columns):
+                text = '?'
+                if content == 'bytes' and item.stat is not None:
+                    text = str(item.stat.st_size)
+                if content == 'size' and item.stat is not None:
+                    text = format_filesize(item.stat.st_size)
+                if content == 'owner' and item.stat is not None and pwd is not None:
+                    uid = item.stat.st_uid
+                    try:
+                        text = pwd.getpwuid(uid)[0]
+                    except:
+                        text = str(uid)
+                if content == 'group' and item.stat is not None and grp is not None:
+                    gid = item.stat.st_gid
+                    try:
+                        text = grp.getgrgid(gid)[0]
+                    except:
+                        text = str(gid)
+                if content in ['num_rights', 'octmode'] and item.stat is not None:
+                    text = oct(stat.S_IMODE(item.stat.st_mode))[2:]
+                if content in ['rights', 'mode'] and item.stat is not None:
+                    text = stat.filemode(item.stat.st_mode)
+                if content == 'lm_abs' and item.stat is not None:
+                    text = format_datetime(datetime.datetime.fromtimestamp(item.stat.st_mtime))
+                    if text is None:
+                        text = '?'
+                if content == 'lm_duration' and item.stat is not None:
+                    text = datetime.datetime.now() - \
+                           datetime.datetime.fromtimestamp(item.stat.st_mtime)
+                    text = duration_as_str(text)
+                text = ' ' + text
+                self.info_columns_content[item.path][column] = text
+        for column in range(len(self.info_columns)):
+            widths = {len(c[column])
+                      for c in self.info_columns_content.values()}
+            self.info_columns_width[column] = max(widths)
+
+
+def duration_as_str(duration):
+    seconds = int(duration.total_seconds())
+    days = seconds // 86400
+    hours = (seconds % 86400) // 3600
+    minutes = (seconds % 3600) // 60
+    
+    result = ''
+    if days > 0:
+        result += f'{days}d'
+    if hours > 0:
+        result += f'{hours}h'
+    if minutes > 0:
+        result += f'{minutes}m'
+    if days + hours == 0:
+        result += f'{seconds % 60}s'
+
+    return result
+
 
 @command.registered_command
 class GoToParent(command.Command):
@@ -746,7 +902,7 @@ def shell_command(context, *args):
     scr = context.application.screen
     with ShellContext(scr, True):
         try:
-            subprocess.run(args, shell=True, check=False)
+            subprocess.run(args, shell=False, check=False)
         except (KeyboardInterrupt,):
             pass
 
