@@ -11,6 +11,7 @@ import pathlib
 import traceback
 import importlib
 import queue
+import threading
 import configparser
 import urllib.request
 
@@ -41,6 +42,7 @@ from .detailpanel import DetailPanel as _
 from .commandinput import CommandInput, expand_part
 from .keyhelppanel import KeyHelpPanel
 from .editorpanel import EditorPanel
+from .helppanel import HelpPanel
 from .clipboard import Clipboard
 
 
@@ -101,6 +103,14 @@ class YesNoConfirmation(InputLine):
             super().handle_key(key)
 
 
+class MetaIndexManagerConfiguration(metaindex.configuration.BaseConfiguration):
+    @property
+    def find_is_case_sensitive(self):
+        return self.bool(shared.ALL_SCOPE,
+                         shared.PREF_FIND_CASE_SENSITIVE,
+                         shared.DEFAULT_FIND_CASE_SENSITIVE)
+
+
 class Application(cursedspace.Application):
     def __init__(self, args):
         super().__init__()
@@ -115,9 +125,14 @@ class Application(cursedspace.Application):
         self.keys = [
             (shared.ANY_SCOPE, ('q', ), ('close',)),
             (shared.ANY_SCOPE, ('X', ), ('quit',)),
+            (shared.ANY_SCOPE, ('?', ), ('help',)),
+            (shared.ANY_SCOPE, ('<F1>', ), ('help',)),
             (FilePanel.SCOPE, (Key.RIGHT,), ('open',)),
+            (FilePanel.SCOPE, ('O',), (':open-with',)),
             (DocPanel.SCOPE, (Key.RIGHT,), ('open',)),
+            (FilePanel.SCOPE, ('O',), (':open-with',)),
             (EditorPanel.SCOPE, (Key.RIGHT,), ('open',)),
+            (FilePanel.SCOPE, ('O',), (':open-with',)),
             (FilePanel.SCOPE, (Key.LEFT,), ('go-to-parent',)),
             (shared.ANY_SCOPE, (Key.TAB, ), ('next-panel',)),
             (shared.ANY_SCOPE, ('g', 'n', 'f'), ('new-file-panel',)),
@@ -128,7 +143,6 @@ class Application(cursedspace.Application):
             (shared.ANY_SCOPE, ('g', 'l'), ('go-to-location',)),
             (DocPanel.SCOPE, ('g', 's'), (':search',)),
             (DocPanel.SCOPE, (Key.F3, ), (':search',)),
-            (DocPanel.SCOPE, ('/', ), (':search',)),
             (FilePanel.SCOPE, ('g', 'h'), ('::cd ~', "Go home")),
             (FilePanel.SCOPE, ('g', 'd'), ('details',)),
             (DocPanel.SCOPE, ('g', 'd'), ('details',)),
@@ -165,11 +179,18 @@ class Application(cursedspace.Application):
             (FilePanel.SCOPE, ('v', ), ('invert-selection',)),
             (shared.ANY_SCOPE, ("'", ), ('::jump-to-mark',)),
             (shared.ANY_SCOPE, (Key.DOWN,), ('next-item',)),
+            (shared.ANY_SCOPE, ('j',), ('next-item',)),
             (shared.ANY_SCOPE, (Key.UP,), ('previous-item',)),
+            (shared.ANY_SCOPE, ('k',), ('previous-item',)),
             (shared.ANY_SCOPE, (Key.PGUP,), ('previous-page',)),
             (shared.ANY_SCOPE, (Key.PGDN,), ('next-page',)),
             (shared.ANY_SCOPE, (Key.HOME,), ('go-to-start',)),
             (shared.ANY_SCOPE, (Key.END,), ('go-to-end',)),
+            (shared.ANY_SCOPE, ('g', 'g'), ('go-to-start',)),
+            (shared.ANY_SCOPE, ('G',), ('go-to-end',)),
+            (shared.ANY_SCOPE, ('/',), (':find',)),
+            (shared.ANY_SCOPE, ('n',), ('find-next',)),
+            (shared.ANY_SCOPE, ('N',), ('find-prev',)),
             (EditorPanel.SCOPE, (Key.RETURN,), ('edit-mode',)),
             (EditorPanel.SCOPE, ('i',), ('edit-mode',)),
             (EditorPanel.SCOPE, ('o',), ('add-value',)),
@@ -196,7 +217,7 @@ class Application(cursedspace.Application):
         conf.read_dict({shared.ALL_SCOPE: {PREF_OPENER: DEFAULT_OPENER,
                                            PREF_INFO_TIMEOUT: DEFAULT_INFO_TIMEOUT,
                                            shared.PREF_BORDER: shared.DEFAULT_BORDER}})
-        self.configuration = metaindex.configuration.BaseConfiguration(conf)
+        self.configuration = MetaIndexManagerConfiguration(conf)
         utils.parse_ls_colors()
         utils.parse_ls_icons()
 
@@ -247,6 +268,9 @@ class Application(cursedspace.Application):
         self.cache = MemoryCache(self.metaindexconf)
         self.cache.start()
 
+        self.known_programs = []
+        self.refresh_known_programs()
+
         configpath = pathlib.Path(args.config).expanduser().resolve()
         self.load_config_file(configpath)
 
@@ -424,7 +448,9 @@ class Application(cursedspace.Application):
         if isinstance(command, str):
             command = resolve_command(command)
 
-        if type(command).__name__ == 'type' and issubclass(command, Command):
+        if command is not None and \
+           type(command).__name__ == 'type' and \
+           issubclass(command, Command):
             command = command()
 
         if command is None:
@@ -824,8 +850,13 @@ class Application(cursedspace.Application):
         if len(opener) == 0:
             # TODO: check for existing file
             return
+        self.open_with(path, opener)
+
+    def open_with(self, path, cmd):
+        use_shell = len(cmd) == 0
+
         with ShellContext(self.screen):
-            subprocess.run(opener + [str(path)])
+            subprocess.run(cmd + [str(path)], shell=use_shell, check=True)
         self.paint(True)
         self.set_term_title(shared.PROGRAMNAME)
 
@@ -901,6 +932,27 @@ class Application(cursedspace.Application):
         self.current_panel.win.noutrefresh()
         curses.doupdate()
 
+    def refresh_known_programs(self):
+        threading.Thread(target=self._do_refresh_known_programs).start()
+
+    def _do_refresh_known_programs(self):
+        paths = [pathlib.Path(p)
+                 for p in os.getenv('PATH', os.defpath).split(os.pathsep)]
+        programs = set()
+        for path in paths:
+            try:
+                if not path.is_dir() or not os.access(path, os.X_OK):
+                    continue
+            except OSError:
+                continue
+            for item in path.iterdir():
+                try:
+                    if item.is_file() and os.access(item, os.F_OK | os.X_OK):
+                        programs.add(item)
+                except OSError:
+                    pass
+        self.known_programs = list(sorted(programs))
+
     @staticmethod
     def as_printable(value, shorten=100):
         """Clean up a string to make it printable
@@ -1007,7 +1059,7 @@ def run():
 
     if ADDONSPATH.exists():
         prev_sys_path = sys.path.copy()
-        sys.path = [str(ADDONSPATH)]
+        sys.path = [str(ADDONSPATH)] + sys.path
         for item in ADDONSPATH.iterdir():
             if item.is_file() and item.suffix == '.py':
                 importlib.import_module(item.stem)
