@@ -187,6 +187,7 @@ class ListPanel(ScrollPanel):
         self.reload_keybindings()
         self._is_busy = threading.Lock()
         self.multi_selection = []
+        self.find_text = None
 
     def reload_keybindings(self):
         self.SCROLL_NEXT = extract_key_sequence(self.app, NEXT_ITEM)
@@ -272,6 +273,65 @@ class ListPanel(ScrollPanel):
     def on_focus_lost(self):
         pass
 
+    def filter(self, text):
+        """Filter the list of items to the one matching the filter text"""
+
+    def find(self, text):
+        """Find the first occurrence of 'text' in the items
+
+        Start searching after the current line"""
+        self.find_text = text
+        if not self.find_next():
+            self.app.info("No match found")
+
+    def line_matches_find(self, cursor):
+        """Return True if the line at cursor matches the find_text"""
+        if not isinstance(self.items[cursor], str):
+            raise RuntimeError("Cannot find in this panel")
+
+        if self.find_text is None:
+            return False
+
+        if self.app.configuration.find_is_case_sensitive:
+            return self.find_text in self.items[cursor]
+        return self.find_text.lower() in self.items[cursor].lower()
+
+    def find_next(self):
+        """Jump to the next occurrence of the find_text
+
+        Return False if no match could be found, otherwise True"""
+        if self.find_text is None:
+            return False
+
+        idx = self.cursor
+        for idx in range(1, len(self.items)):
+            pos = (self.cursor + idx) % len(self.items)
+            if self.line_matches_find(pos):
+                self.cursor = pos
+
+                must_repaint = self.scroll()
+                self.paint(must_repaint)
+                return True
+        return False
+
+    def find_previous(self):
+        """Jump to the previous occurrence of the find_text
+
+        Return False if no match could be found, otherwise True"""
+        if self.find_text is None:
+            return False
+
+        idx = self.cursor
+        for idx in range(1, len(self.items)):
+            pos = (self.cursor - idx) % len(self.items)
+            if self.line_matches_find(pos):
+                self.cursor = pos
+
+                must_repaint = self.scroll()
+                self.paint(must_repaint)
+                return True
+        return False
+
 
 def extract_key_sequence(application, commandname):
     """Get all key sequences of this command in the 'any' scope
